/* Copyright (C) 1998-9 Pancrazio `Ezio' de Mauro <p@demauro.net>
 * Copyright (C) 2004 Milosz `Dem' Tanski <mtanski@gmail.com>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version
 * 2 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * $Id: zengarden.c,v 1.9 2004/12/20 01:36:35 demie Exp $
 *
 * -----------------------------------------------------------------
 * zengarden.h: libc wrapper functions
 */

#include <sys/param.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/mman.h>

#include <stdarg.h>
#include <stdio.h>
#include <stdlib.h>
#include <syslog.h>
#include <errno.h>
#include <unistd.h>
#include <string.h>
#include <time.h>

#include <fcntl.h>
#include <dlfcn.h>

#include "zengarden.h"

time_t time (time_t *timer) {
  return true_time(timer);
}

int chmod(const char *path, mode_t mode) {
  int result;
  char *canonic = canonicalize(path);

  check_jail(path, "chmod");
  backup (canonic);
 
  result = true_chmod(path, mode);
  logger("%d\tchmod\t%s\t0%04o\t#%s\n", result, canonic, mode, error(result));

  free(canonic);
  return result;
}

int chown(const char *path, uid_t owner, gid_t group) {
  int result;
  char *canonic = canonicalize(path);

  check_jail(path, "chown");
  backup (canonic);

  result = true_chown(path, owner, group);
  logger("%d\tchown\t%s\t%d\t%d\t#%s\n", result, canonic, owner, group, error(result));

  free(canonic);
  return result;
}

int chroot(const char *path) {
  int result = true_chroot(path);;
  char *canonic = canonicalize(path);

  /* chroot in a make jail, will this ever happen */
  check_jail(path, "chroot");

  /* Fromnow on, another log file will be written if *
   * INSTALLWATCHFILE is set                          */
  logger("%d\tchroot\t%s\t#%s\n", result, canonic, error(result));

  return result;
}

int creat(const char *pathname, mode_t mode) {
  int result;
  char *canonic = canonicalize(pathname);

  check_jail(pathname, "creat");
  backup(pathname);
  
  result = true_open(pathname, O_CREAT | O_WRONLY | O_TRUNC, mode);
  logger("%d\tcreat\t%s\t#%s\n", result, canonic, error(result));

  free(canonic);
  return result;
}

int fchmod(int filedes, mode_t mode) {
  int result;
 
  result = true_fchmod(filedes, mode);
  logger("%d\tfchmod\t%d\t0%04o\t#%s\n", result, filedes, mode, error(result));

  return result;
}

int fchown(int fd, uid_t owner, gid_t group) {
  int result;
  
  result = true_fchown(fd, owner, group);
  logger("%d\tfchown\t%d\t%d\t%d\t#%s\n", result, fd, owner, group, error(result));
  
  return result;
}

FILE *fopen(const char *pathname, const char *mode) {
  FILE *result;
  char *canonic = canonicalize(pathname);

  if(mode[0]=='w'||mode[0]=='a'||mode[1]=='+') {
    check_jail(pathname, "fopen");
    backup (canonic);
  }
  
  result = true_fopen(pathname,mode);
  if(mode[0]=='w'||mode[0]=='a'||mode[1]=='+') 
    logger("%d\tfopen\t%s\t#%s\n", (int)result, canonic, error(result));

  free(canonic);
  return result;
}

int ftruncate(int fd, off_t length) {
  int result;

  result = true_ftruncate(fd, length);
  logger("%d\tftruncate\t%d\t%d\t#%s\n", result, fd, (int)length, error(result));

  return result;
}

int lchown(const char *path, uid_t owner, gid_t group) {
  int result;
  char *canonic = canonicalize(path);

  check_jail(path, "lchown");
  backup(canonic);

  result = true_chown(path, owner, group);
  logger("%d\tlchown\t%s\t%d\t%d\t#%s\n", result, canonic, owner, group, error(result));
  
  free(canonic);
  return result;
}

int link(const char *oldpath, const char *newpath) {
  int result;
  char *old_canonic = canonicalize(oldpath);
  char *new_canonic = canonicalize(newpath);

  check_jail(newpath, "link");

  result = true_link(oldpath, newpath);
  logger("%d\tlink\t%s\t%s\t#%s\n", result, old_canonic, new_canonic, error(result));
  
  free(old_canonic);
  free(new_canonic);
  return result;
}

int mkdir(const char *pathname, mode_t mode) {
  int result;
  char *canonic = canonicalize(pathname);
 
  check_jail(pathname, "mkdir");
 
  result = true_mkdir(pathname, mode);
  logger("%d\tmkdir\t%s\t#%s\n", result, canonic, error(result));
  
  free(canonic);
  return result;
}

int open(const char *pathname, int flags, ...) {
/* Eventually, there is a third parameter: it's mode_t mode */
  va_list ap;
  mode_t mode;
  int result;
  char *canonic = canonicalize(pathname);

  va_start(ap, flags);
  mode = va_arg(ap, mode_t);
  va_end(ap);

  if(flags & (O_WRONLY | O_RDWR)) {
    check_jail(pathname, "open");
    backup (canonic);
  }
    
  result = true_open(pathname, flags, mode);
  if(flags & (O_WRONLY | O_RDWR)) 
    logger("%d\topen\t%s\t#%s\n", result, canonic, error(result));

  free(canonic);
  return result;
}

int rename(const char *oldpath, const char *newpath) {
  int result;
  char *old_canonic = canonicalize(oldpath);
  char *new_canonic = canonicalize(newpath);

  backup (old_canonic);

  result = true_rename(oldpath, newpath);
  logger("%d\trename\t%s\t%s\t#%s\n", result, old_canonic, new_canonic, error(result));
  
  free(old_canonic);
  free(new_canonic);
  return result;
}

int rmdir(const char *pathname) {
  int result;
  char *canonic = canonicalize(pathname);

  check_jail(pathname, "rmdir");
  backup(canonic);

  result = true_rmdir(pathname);
  logger("%d\trmdir\t%s\t#%s\n", result, canonic, error(result));

  free(canonic);
  return result;
}

int symlink(const char *oldpath, const char *newpath) {
  int result;
  char *old_canonic = canonicalize(oldpath);
  char *new_canonic = canonicalize(newpath);

  result = true_symlink(oldpath, newpath);
  logger("%d\tsymlink\t%s\t%s\t#%s\n", result, old_canonic, new_canonic, error(result));

  free(old_canonic);
  free(new_canonic);
  return result;
}

int truncate(const char *path, off_t length) {
  int result;
  char *canonic = canonicalize(path);

  check_jail(path, "truncate");
  backup (canonic);

  result = true_truncate(path, length);
  logger("%d\ttruncate\t%s\t%d\t#%s\n", result, path, (int)length, error(result));
  
  free(canonic);
  return result;
}

int unlink(const char *pathname) {
  int result;
  char *canonic = canonicalize(pathname);
 
  check_jail(pathname, "unlink");
  backup(canonic);

  result = true_unlink(pathname);
  logger("%d\tunlink\t%s\t#%s\n", result, canonic, error(result));
 
  free(canonic);
  return result;
}

int creat64(const char *pathname, __mode_t mode) {
/* Is it a system call? */
  int result;
  char *canonic = canonicalize(pathname);

  check_jail(pathname, "creat");
  backup(canonic);

  result = true_open64(pathname, O_CREAT | O_WRONLY | O_TRUNC, mode);
  logger("%d\tcreat\t%s\t#%s\n", result, canonic, error(result));

  free(canonic);
  return result;
}

int ftruncate64(int fd, __off64_t length) {
  int result;

  result = true_ftruncate64(fd, length);
  logger("%d\tftruncate\t%d\t%d\t#%s\n", result, fd, (int)length, error(result));
  
  return result;
}

FILE *fopen64(const char *pathname, const char *mode) {
  FILE *result;
  char *canonic = canonicalize(pathname);

  if(mode[0]=='w'||mode[0]=='a'||mode[1]=='+') {
    check_jail(pathname, "fopen");
    backup(canonic);
  }

  result = true_fopen(pathname,mode);
  if(mode[0]=='w'||mode[0]=='a'||mode[1]=='+') 
     logger("%d\tfopen64\t%s\t#%s\n", (int)result, canonic, error(result));

  free(canonic);
  return result;
}

int open64(const char *pathname, int flags, ...) {
/* Eventually, there is a third parameter: it's mode_t mode */
  va_list ap;
  mode_t mode;
  int result;
  char *canonic = canonicalize(pathname);

  va_start(ap, flags);
  mode = va_arg(ap, mode_t);
  va_end(ap);

  if(flags & (O_WRONLY | O_RDWR)) {
    check_jail(pathname, "open");
    backup(canonic);
  }
    
  result = true_open64(pathname, flags, mode);
  
  if(flags & (O_WRONLY | O_RDWR)) 
    logger("%d\topen\t%s\t#%s\n", result, canonic, error(result));

  free(canonic);
  return result;
}

int truncate64(const char *path, __off64_t length) {
  int result;
  char *canonic = canonicalize(path);

  backup(canonic);

  result = true_truncate64(path, length);
  logger("%d\ttruncate\t%s\t%d\t#%s\n", result, path, (int)length, error(result));
 
  free(canonic);
  return result;
}
// vim:tabstop=2:shiftwidth=2:expandtab:
